@@ -17,24 +17,25 @@ class Path:
     command_lengths = {'M': 2, 'm': 2, 'L': 2, 'l': 2, 'H': 1, 'h': 1, 'V': 1, 'v': 1, 'Z': 0, 'z': 0, 'C': 6, 'c': 6,
                        'Q': 4, 'q': 4, 'S': 4, 's': 4, 'T': 2, 't': 2, 'A': 7, 'a': 7}
 
-    __slots__ = "curves", "start", "end", "last_control", "canvas_height", "do_vertical_mirror", \
-                "do_vertical_translate", "draw_move"
+    __slots__ = "curves", "initial_point", "current_point", "last_control", "canvas_height", "do_vertical_mirror", \
+                "do_vertical_translate", "draw_move", "transformation"
 
-    def __init__(self, d: str, canvas_height: float, do_vertical_mirror=True, do_vertical_translate=True):
+    def __init__(self, d: str, canvas_height: float, do_vertical_mirror=True, do_vertical_translate=True,
+                 transformation=None):
         self.canvas_height = canvas_height
         self.do_vertical_mirror = do_vertical_mirror
         self.do_vertical_translate = do_vertical_translate
+        self.transformation = transformation
 
         self.curves = []
-        self.start = None  # type: Vector
-        self.end = Vector(0, 0)
+        self.initial_point = Vector(0, 0)  # type: Vector
+        self.current_point = Vector(0, 0)
         self.last_control = None  # type: Vector
 
-        self._parse_commands(d)
         try:
-            pass
-            #self._parse_commands(d)
+            self._parse_commands(d)
         except Exception as generic_exception:
+            raise generic_exception
             warnings.warn(f"Terminating path. The following unforeseen exception occurred: {generic_exception}")
 
     def __repr__(self):
@@ -105,10 +106,13 @@ class Path:
 
     def _transform_coordinate_system(self, point: Vector):
         """
-        If both do_vertical_mirror and do_vertical_translate are true, it will transform a point form a coordinate
+        If both do_vertical_mirror and do_vertical_translate are true, it will transform_origin a point form a coordinate
         system with the origin at the top-left, to one with origin at the bottom-right.
         """
 
+        if self.transformation:
+            point = self.transformation.apply_transformation(point)
+
         if self.do_vertical_mirror:
             point = Vector(point.x, -point.y)
 
@@ -124,56 +128,59 @@ class Path:
 
         Each sub-method must be implemented with the following structure:
         def descriptive_name(*command_arguments):
-            execute calculations, **do not modify or create any instance variables**
+            execute calculations and transformations, **do not modify or create any instance variables**
             generate curve
             modify instance variables
             return curve
 
+        Alternatively a sub-method may simply call a base command.
+
         :param command_key: a character representing a specific command based on the svg standard
         :param command_arguments: A list containing the arguments for the current command_key
         """
 
-        # Only move end point
+        # Establish a new initial point and a new current point. (multiple coordinates are parsed as lineto commands)
         def absolute_move(x, y):
-            self.end = Vector(x, y)
+            self.initial_point = Vector(x, y)
+            self.current_point = Vector(x, y)
             return None
 
         def relative_move(dx, dy):
-            return absolute_move(*(self.end + Vector(dx, dy)))
+            return absolute_move(*(self.current_point + Vector(dx, dy)))
 
         # Draw straight line
         def absolute_line(x, y):
-            start = self.end
+            start = self.current_point
             end = Vector(x, y)
 
             line = Line(self._transform_coordinate_system(start), self._transform_coordinate_system(end))
 
-            self.end = end
+            self.current_point = end
 
             return line
 
         def relative_line(dx, dy):
-            return absolute_line(*(self.end + Vector(dx, dy)))
+            return absolute_line(*(self.current_point + Vector(dx, dy)))
 
         def absolute_horizontal_line(x):
-            return absolute_line(x, self.end.y)
+            return absolute_line(x, self.current_point.y)
 
         def relative_horizontal_line(dx):
-            return absolute_horizontal_line(self.end.x + dx)
+            return absolute_horizontal_line(self.current_point.x + dx)
 
         def absolute_vertical_line(y):
-            return absolute_line(self.end.x, y)
+            return absolute_line(self.current_point.x, y)
 
         def relative_vertical_line(dy):
-            return absolute_vertical_line(self.end.y + dy)
+            return absolute_vertical_line(self.current_point.y + dy)
 
         def close_path():
-            return absolute_line(*self.start)
+            return absolute_line(*self.initial_point)
 
-        # Draw Curves
+        # Draw curvy curves
         def absolute_cubic_bazier(control1_x, control1_y, control2_x, control2_y, x, y):
 
-            trans_start = self._transform_coordinate_system(self.end)
+            trans_start = self._transform_coordinate_system(self.current_point)
             trans_end = self._transform_coordinate_system(Vector(x, y))
             trans_control1 = self._transform_coordinate_system(Vector(control1_x, control1_y))
             trans_control2 = self._transform_coordinate_system(Vector(control2_x, control2_y))
@@ -181,17 +188,17 @@ class Path:
             cubic_bezier = CubicBazier(trans_start, trans_end, trans_control1, trans_control2)
 
             self.last_control = Vector(control2_x, control2_y)
-            self.end = Vector(x, y)
+            self.current_point = Vector(x, y)
 
             return cubic_bezier
 
         def relative_cubic_bazier(dx1, dy1, dx2, dy2, dx, dy):
-            return absolute_cubic_bazier(self.end.x + dx1, self.end.y + dy1,
-                                         self.end.x + dx2, self.end.y + dy2,
-                                         self.end.x + dx, self.end.y + dy)
+            return absolute_cubic_bazier(self.current_point.x + dx1, self.current_point.y + dy1,
+                                         self.current_point.x + dx2, self.current_point.y + dy2,
+                                         self.current_point.x + dx, self.current_point.y + dy)
 
         def absolute_cubic_bezier_extension(x2, y2, x, y):
-            start = self.end
+            start = self.current_point
             control2 = Vector(x2, y2)
             end = Vector(x, y)
 
@@ -201,33 +208,33 @@ class Path:
             else:
                 bazier = absolute_quadratic_bazier(*control2, *end)
 
-            self.end = start
+            self.current_point = start
 
             return bazier
 
         def relative_cubic_bazier_extension(dx2, dy2, dx, dy):
-            return absolute_cubic_bezier_extension(self.end.x + dx2, self.end.y + dy2,
-                                                   self.end.x + dx, self.end.y + dy)
+            return absolute_cubic_bezier_extension(self.current_point.x + dx2, self.current_point.y + dy2,
+                                                   self.current_point.x + dx, self.current_point.y + dy)
 
         def absolute_quadratic_bazier(control1_x, control1_y, x, y):
 
-            trans_end = self._transform_coordinate_system(self.end)
+            trans_end = self._transform_coordinate_system(self.current_point)
             trans_new_end = self._transform_coordinate_system(Vector(x, y))
             trans_control1 = self._transform_coordinate_system(Vector(control1_x, control1_y))
 
             quadratic_bezier = QuadraticBezier(trans_end, trans_new_end, trans_control1)
 
             self.last_control = Vector(control1_x, control1_y)
-            self.end = Vector(x, y)
+            self.current_point = Vector(x, y)
 
             return quadratic_bezier
 
         def relative_quadratic_bazier(dx1, dy1, dx, dy):
-            return absolute_quadratic_bazier(self.end.x + dx1, self.end.y + dy1,
-                                             self.end.x + dx, self.end.y + dy)
+            return absolute_quadratic_bazier(self.current_point.x + dx1, self.current_point.y + dy1,
+                                             self.current_point.x + dx, self.current_point.y + dy)
 
         def absolute_quadratic_bazier_extension(x, y):
-            start = self.end
+            start = self.current_point
             end = Vector(x, y)
 
             if self.last_control:
@@ -236,20 +243,21 @@ class Path:
             else:
                 bazier = absolute_quadratic_bazier(*start, *end)
 
-            self.end = end
+            self.current_point = end
             return bazier
 
         def relative_quadratic_bazier_extension(dx, dy):
-            return absolute_quadratic_bazier_extension(self.end.x + dx, self.end.y + dy)
+            return absolute_quadratic_bazier_extension(self.current_point.x + dx, self.current_point.y + dy)
 
         # Generate EllipticalArc with center notation from svg endpoint notation.
         # Based on w3.org implementation notes. https://www.w3.org/TR/SVG2/implnote.html
         def absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, x, y):
-
-            start = self.end
+            start = self.current_point
             end = Vector(x, y)
 
             rotation_rad = math.radians(deg_from_horizontal)
+            max_angle = 2 * math.pi
+            rotation_rad = formulas.mod_constrain(rotation_rad, -max_angle, max_angle)
 
             # Find and select one of the two possible eclipse centers by undoing the rotation (to simplify the math) and
             # then re-applying it.
@@ -258,14 +266,13 @@ class Path:
             px, py = primed_values.x, primed_values.y
 
             # Correct out-of-range radii
-            # ToDo investigate buggy behaviour
+            # ToDo investigate buggy behaviour when sweep angle > 180 deg
             rx = abs(rx)
             ry = abs(ry)
             if rx <= TOLERANCES['operation'] or ry <= TOLERANCES['operation']:
                 return absolute_line(x, y)
 
             delta = px**2/rx**2 + py**2/ry**2
-            print(delta, rx, ry)
 
             if delta > 1:
                 rx *= math.sqrt(delta)
@@ -288,17 +295,27 @@ class Path:
             v = Vector((-px - cx) / rx, (-py - cy) / ry)
 
             start_angle = formulas.angle_between_vectors(Vector(1, 0), u)
-            sweep_angle = formulas.angle_between_vectors(u, v)
+            sweep_angle_unbounded = formulas.angle_between_vectors(u, v)
+            sweep_angle = sweep_angle_unbounded % max_angle
 
-            arc = EllipticalArc(self._transform_coordinate_system(rotated_center), Vector(rx, -ry),
-                                rotation_rad, start_angle, sweep_angle)
+            if not sweep_flag and sweep_angle_unbounded > 0:
+                sweep_angle -= max_angle
 
-            self.end = Vector(x, y)
+            if sweep_flag and sweep_angle_unbounded < 0:
+                sweep_angle += max_angle
+
+            transformed_center = self._transform_coordinate_system(rotated_center)
+            sweep_angle *= -1 if self.do_vertical_mirror else 1
+            start_angle *= 1 if self.do_vertical_mirror else 1
+
+            arc = EllipticalArc(transformed_center, Vector(rx, ry), rotation_rad, start_angle, sweep_angle)
+
+            self.current_point = Vector(x, y)
 
             return arc
 
         def relative_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, dx, dy):
-            return absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, dx, dy)
+            return absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, self.current_point.x + dx, self.current_point.x + dy)
 
         command_methods = {
             # Only move end point
@@ -329,11 +346,11 @@ class Path:
             'A': absolute_arc,
             'a': relative_arc
         }
-        curve = command_methods[command_key](*command_arguments)
+
         try:
-            pass
-            #curve = command_methods[command_key](*command_arguments)
+            curve = command_methods[command_key](*command_arguments)
         except TypeError as type_error:
+            raise type_error
             warnings.warn(f"Mis-formed input. Skipping command {command_key, command_arguments} because it caused the "
                           f"following error: \n{type_error}")
         except ValueError as value_error:
@@ -343,8 +360,5 @@ class Path:
             if curve is not None:
                 self.curves.append(curve)
 
-        if self.start is None:
-            self.start = Vector(*self.end)
-
         if verbose:
             print(f"{command_key}{tuple(command_arguments)} -> {curve}")
